import { Injectable } from '@nestjs/common';
import { EntityManager } from '@mikro-orm/core';
import { InjectRepository } from '@mikro-orm/nestjs';
import { EntityRepository } from '@mikro-orm/mysql';
import { RoomSessionUser } from './roomSessionUser.entity';
import { RoomSession } from './roomSession.entity';
import { RoomSessionUserPointsResponseDto } from './roomSessionUser.dto';
import { SessionPoints } from '../points/sessionPoints.entity';
import { ActivitiesService } from '../activities/activities.service';

/**
 * A service responsible for managing room sessions and users.
 */
@Injectable()
export class RoomSessionUserService {
    /**
     * Initializes the service with a manager and repository.
     * @param em - The EntityManager instance provided by MikroORM.
     * @param repository - The repository for the Room entity.
     */
    constructor(
        private readonly em: EntityManager,
        @InjectRepository(SessionPoints)
        private readonly pointsRepository: EntityRepository<SessionPoints>,
        @InjectRepository(RoomSessionUser)
        private readonly repository: EntityRepository<RoomSessionUser>,
        private readonly activitiesService: ActivitiesService,
    ) {}

    public async addSessionUser(roomSession: RoomSession, userId: string, name: string): Promise<RoomSessionUser> {
        const roomSessionUser = new RoomSessionUser(roomSession, userId, name);
        await this.em.persistAndFlush(roomSessionUser);
        return roomSessionUser;
    }
    
    async updateSessionUserName(roomSession: RoomSession, userId: string, name:string) {
        let user = await this.repository.findOne({ roomSession, userId });
        if (user) {
            user.name = name;
        } else {
            user = new RoomSessionUser(roomSession, userId, name);
        }
        await this.em.persistAndFlush(user);
        return user;
    }

    /**
        * Retrieves detailed points breakdown by activity type for a specific user.
        * 
        * @param userId - The unique identifier of the user whose points are to be retrieved.
        * @returns A promise that resolves to an object containing the detailed points breakdown.
        */
        public async getPointsByRoomSessionUser(userId: number): Promise<RoomSessionUserPointsResponseDto> {
            const pointsEntries = await this.pointsRepository.find({ roomSessionUser: userId });
        
            let totalPoints = 0;
            let audioPoints = 0;
            let videoPoints = 0;
            let browserPoints = 0;
            let whiteboardPoints = 0;
            let notePoints = 0;
        
            pointsEntries.forEach(entry => {
                audioPoints += entry.audioPoints;
                videoPoints += entry.videoPoints;
                browserPoints += entry.browserPoints;
                whiteboardPoints += entry.whiteboardPoints;
                notePoints += entry.notePoints;
                totalPoints += entry.audioPoints + entry.videoPoints + entry.browserPoints + 
                          entry.whiteboardPoints + entry.notePoints;
        });        
            return {
                totalPoints,
                userId,
                allPoints: {
                audioPoints,
                videoPoints,
                browserPoints,
                whiteboardPoints,
                notePoints
                },
            };
        }
}
